import * as React from 'react';
import { styled, alpha } from '@mui/material/styles';
import {Paper} from '@mui/material';
import { ViewState ,EditingState} from '@devexpress/dx-react-scheduler';
import {
  Scheduler,
  WeekView,
  Appointments,
  CurrentTimeIndicator,
  AppointmentTooltip,
  AppointmentForm,
  Toolbar,
  DragDropProvider,
  EditRecurrenceMenu,
  DateNavigator,
  TodayButton
} from '@devexpress/dx-react-scheduler-material-ui';
import { teal } from '@mui/material/colors';
import classNames from 'clsx';
import appointments from './demo-data/today-appointments';
import TwoHourSwitcher from '../../components/UI/WeekComponents/TwoHourSwitcher';

const PREFIX = 'Demo';
const SHIFT_KEY = 16;

const classes = {
  todayCell: `${PREFIX}-todayCell`,
  weekendCell: `${PREFIX}-weekendCell`,
  today: `${PREFIX}-today`,
  weekend: `${PREFIX}-weekend`,
  line: `${PREFIX}-line`,
  circle: `${PREFIX}-circle`,
  nowIndicator: `${PREFIX}-nowIndicator`,
  shadedCell: `${PREFIX}-shadedCell`,
  shadedPart: `${PREFIX}-shadedPart`,
  appointment: `${PREFIX}-appointment`,
  shadedAppointment: `${PREFIX}-shadedAppointment`,
};



const StyledDiv = styled('div', {
    shouldForwardProp: prop => prop !== 'top',
  })(({ theme, top }) => ({
    [`& .${classes.line}`]: {
      height: '2px',
      borderTop: `2px ${theme.palette.primary.main} dotted`,
      width: '100%',
      transform: 'translate(0, -1px)',
    },
    [`& .${classes.circle}`]: {
      width: theme.spacing(1.5),
      height: theme.spacing(1.5),
      borderRadius: '50%',
      transform: 'translate(-50%, -50%)',
      background: theme.palette.primary.main,
    },
    [`& .${classes.nowIndicator}`]: {
      position: 'absolute',
      zIndex: 1,
      left: 0,
      top,
    },
  }));


const StyledWeekViewTimeTableCell = styled(WeekView.TimeTableCell)(({ theme,currentTimeIndicatorPosition }) => ({
  [`&.${classes.todayCell}`]: {
    backgroundColor: alpha(theme.palette.primary.main, 0.1),
    '&:hover': {
      backgroundColor: alpha(theme.palette.primary.main, 0.14),
    },
    '&:focus': {
      backgroundColor: alpha(theme.palette.primary.main, 0.16),
    },
  },
  [`&.${classes.weekendCell}`]: {
    backgroundColor: alpha(theme.palette.action.disabledBackground, 0.04),
    '&:hover': {
      backgroundColor: alpha(theme.palette.action.disabledBackground, 0.04),
    },
    '&:focus': {
      backgroundColor: alpha(theme.palette.action.disabledBackground, 0.04),
    },
  },
  [`&.${classes.shadedCell}`]: {
    backgroundColor: alpha(theme.palette.primary.main, 0.08),
    '&:hover': {
      backgroundColor: alpha(theme.palette.primary.main, 0.12),
    },
    '&:focus': {
      backgroundColor: alpha(theme.palette.primary.main, 0.20),
      outline: 0,
    },
  },
  [`& .${classes.shadedPart}`]: {
    backgroundColor: alpha(theme.palette.primary.main, 0.08),
    position: 'absolute',
    height: currentTimeIndicatorPosition,
    width: '100%',
    left: 0,
    top: 0,
    'td:focus &': {
      backgroundColor: alpha(theme.palette.primary.main, 0.12),
    },
  },
}));

// const StyledWeekViewDayScaleCell = styled(WeekView.DayScaleCell)(({ theme }) => ({
//   [`&.${classes.today}`]: {
//     backgroundColor: alpha(theme.palette.primary.main, 0.16),
//   },
//   [`&.${classes.weekend}`]: {
//     backgroundColor: alpha(theme.palette.action.disabledBackground, 0.06),
//   },
// }));

const StyledAppointmentsAppointment = styled(Appointments.Appointment)(() => ({
    [`&.${classes.appointment}`]: {
      backgroundColor: teal[300],
      '&:hover': {
        backgroundColor: teal[400],
      },
    },
    [`&.${classes.shadedAppointment}`]: {
      backgroundColor: teal[200],
      '&:hover': {
        backgroundColor: teal[300],
      },
    },
  }));

  const StyledToolbarFlexibleSpace = styled(Toolbar.FlexibleSpace)(() => ({
    [`&.${classes.flexibleSpace}`]: {
      flex: 'none',
    },
    [`& .${classes.flexContainer}`]: {
      display: 'flex',
      alignItems: 'center',
    },
  }));
  const StyledAppointmentsAppointmentContent = styled(Appointments.AppointmentContent)(() => ({
    [`&.${classes.apptContent}`]: {
      '&>div>div': {
        whiteSpace: 'normal !important',
        lineHeight: 1.2,
      },
    },
  }));

const TimeIndicator = ({
    top, ...restProps
    // #FOLD_BLOCK
  }) => (
    <StyledDiv top={top} {...restProps}>
      <div className={classNames(classes.nowIndicator, classes.circle)} />
      <div className={classNames(classes.nowIndicator, classes.line)} />
    </StyledDiv>
  );

// const TimeTableCell = (props) => {
//   const { startDate } = props;
//   const date = new Date(startDate);

//   if (date.getDate() === new Date().getDate()) {
//     return <StyledWeekViewTimeTableCell {...props} className={classes.todayCell} />;
//   } if (date.getDay() === 0 || date.getDay() === 6) {
//     return <StyledWeekViewTimeTableCell {...props} className={classes.weekendCell} />;
//   } return <StyledWeekViewTimeTableCell {...props} />;
// };
const TimeTableCell = ({
    currentTimeIndicatorPosition, isShaded, ...restProps
    // #FOLD_BLOCK
  }) => {
    const isNow = !!currentTimeIndicatorPosition;
    return (
      <StyledWeekViewTimeTableCell
        isShaded={isShaded && !isNow}
        currentTimeIndicatorPosition={currentTimeIndicatorPosition}
        className={classNames({
          [classes.shadedCell]: isShaded && !isNow,
        })}
        {...restProps}
      >
        {isNow && isShaded && (
          <div className={classes.shadedPart} />
        )}
      </StyledWeekViewTimeTableCell>
    );
  };

  const Appointment = ({
    isShaded, ...restProps
    // #FOLD_BLOCK
  }) => (
    <StyledAppointmentsAppointment
      className={classNames({
        [classes.appointment]: true,
        [classes.shadedAppointment]: isShaded,
      })}
      {...restProps}
    />
  );
  const AppointmentContent = (({ ...restProps }) => (
    <StyledAppointmentsAppointmentContent {...restProps} className={classes.apptContent} />
  ));

// const DayScaleCell = (props) => {
//   const { startDate, today } = props;

//   if (today) {
//     return <StyledWeekViewDayScaleCell {...props} className={classes.today} />;
//   } if (startDate.getDay() === 0 || startDate.getDay() === 6) {
//     return <StyledWeekViewDayScaleCell {...props} className={classes.weekend} />;
//   } return <StyledWeekViewDayScaleCell {...props} />;
// };

const FlexibleSpace = (({ ...restProps }) => (
  <StyledToolbarFlexibleSpace {...restProps} className={classes.flexibleSpace}>
    <div className={classes.flexContainer}>
    {/* <TwoHourSwitcher/> */}
      {/* <ColorLens fontSize="large" htmlColor="#FF7043" /> */}
      {/* <Typography variant="h5" style={{ marginLeft: '10px' }}>Art School</Typography> */}
    </div>
  </StyledToolbarFlexibleSpace>
));
export default class Demo extends React.PureComponent {
    constructor(props) {
      super(props);
      this.state = {
        data: appointments,
        locale: 'ru-Ru',
        isShiftPressed: false,
      };
      this.commitChanges = this.commitChanges.bind(this);
      this.onKeyDown = this.onKeyDown.bind(this);
      this.onKeyUp = this.onKeyUp.bind(this);
    }


    componentDidMount() {
      window.addEventListener('keydown', this.onKeyDown);
      window.addEventListener('keyup', this.onKeyUp);
    }
  
    componentWillUnmount() {
      window.removeEventListener('keydown');
      window.removeEventListener('keyup');
    }
  
    onKeyDown(event) {
      if (event.keyCode === SHIFT_KEY) {
        this.setState({ isShiftPressed: true });
      }
    }
  
    onKeyUp(event) {
      if (event.keyCode === SHIFT_KEY) {
        this.setState({ isShiftPressed: false });
      }
    }


    commitChanges({ added, changed, deleted }) {
      this.setState((state) => {
        let { data } = state;
        const { isShiftPressed } = this.state;
        if (added) {
          const startingAddedId = data.length > 0 ? data[data.length - 1].id + 1 : 0;
          data = [...data, { id: startingAddedId, ...added }];
        }
        // if (changed) {
        //   data = data.map(appointment => (
        //     changed[appointment.id] ? { ...appointment, ...changed[appointment.id] } : appointment));
        // }
        if (changed) {
          if (isShiftPressed) {
            const changedAppointment = data.find(appointment => changed[appointment.id]);
            const startingAddedId = data.length > 0 ? data[data.length - 1].id + 1 : 0;
            data = [
              ...data,
              { ...changedAppointment, id: startingAddedId, ...changed[changedAppointment.id] },
            ];
          } else {
            data = data.map(appointment => (
              changed[appointment.id]
                ? { ...appointment, ...changed[appointment.id] }
                : appointment));
          }
        }

        if (deleted !== undefined) {
          data = data.filter(appointment => appointment.id !== deleted);
        }
        return { data };
      });
    }
  
    render() {
      const { data ,locale} = this.state;
  
      return (
        <div>
        <Paper>
          <Scheduler
          firstDayOfWeek={1}
            data={data}
            locale={locale}
            height={660}
          >
          <EditingState
            onCommitChanges={this.commitChanges}
          />
             <ViewState
            defaultCurrentDate="2022-11-10"
          />
            <WeekView
              startDayHour={9}
              endDayHour={19}
              timeTableCellComponent={TimeTableCell}
              cellDuration={60}
            />
            <Toolbar
            flexibleSpaceComponent={FlexibleSpace}
             />
            <DateNavigator/>
            <TodayButton/>
            <EditRecurrenceMenu />
            <TwoHourSwitcher/>

            <Appointments
              appointmentComponent={Appointment}
              appointmentContentComponent={AppointmentContent}
            />
                        <DragDropProvider 
                          scrollSpeed={15}
                        />
            

              <AppointmentTooltip
            showCloseButton
            showDeleteButton
            showOpenButton
          />
          <AppointmentForm
            // readOnly
          />
            <CurrentTimeIndicator
              indicatorComponent={TimeIndicator}
              shadePreviousCells
              shadePreviousAppointments
            />
          </Scheduler>
        </Paper>
        </div>
      );
    }
  }
  